#include "CapabilityInterface.h"

//========================================================================
CapabilityInterface::CapabilityInterface(TwainInterface *pTwainInterface)
{
	m_pTwainInterface	= pTwainInterface;
}

//========================================================================
CapabilityInterface::~CapabilityInterface(void)
{
	;
}
//========================================================================
BOOL CapabilityInterface::GetFrame(TW_UINT16 cap, TW_FRAME *twFrame)
{
	BOOL				bEnable		= TRUE;
	TW_UINT16			twrc		= TWRC_SUCCESS;
	TW_CAPABILITY		twCap;

	twCap.Cap			= cap;
	twCap.ConType		= TWON_DONTCARE16;
	twCap.hContainer	= NULL;

	twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
										DAT_CAPABILITY,
										MSG_GET,
										(TW_MEMREF)&twCap,
										FALSE);

	if(TWRC_SUCCESS != twrc)
		return FALSE;

	switch(twCap.ConType)
	{
		case TWON_ONEVALUE:
		{
			pTW_ONEVALUE	pOneValue	= NULL;

			pOneValue = (pTW_ONEVALUE)m_pTwainInterface->DSM_Lock((TW_HANDLE)twCap.hContainer);

			if(twCap.hContainer)
			{
				if(TWTY_FRAME == pOneValue->ItemType)
				{
					pTW_FRAME pFrame = (pTW_FRAME)&pOneValue->Item;
					twFrame->Top	= pFrame->Top;
					twFrame->Left	= pFrame->Left;
					twFrame->Bottom = pFrame->Bottom;
					twFrame->Right	= pFrame->Right;
				}
				m_pTwainInterface->DSM_Unlock((TW_HANDLE)twCap.hContainer);
			}
			break;
		}
		case TWON_ENUMERATION:
		case TWON_ARRAY:
		default:
		{
			bEnable = FALSE;
			break;
		}
	}

	m_pTwainInterface->DSM_Free((TW_HANDLE)twCap.hContainer);

	return bEnable;
}

//========================================================================
BOOL CapabilityInterface::GetValue(TW_UINT16 cap, TW_UINT32 *value)
{
	BOOL				bEnable		= TRUE;
	TW_UINT16			twrc		= TWRC_SUCCESS;
	TW_CAPABILITY		twCap;

	twCap.Cap			= cap;
	twCap.ConType		= TWON_DONTCARE16;
	twCap.hContainer	= NULL;

	twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
										DAT_CAPABILITY,
										MSG_GET,
										(TW_MEMREF)&twCap,
										FALSE);

	if(TWRC_SUCCESS != twrc)
		return FALSE;

	switch(twCap.ConType)
	{
		case TWON_ONEVALUE:
		{
			pTW_ONEVALUE	pOneValue	= NULL;

			pOneValue = (pTW_ONEVALUE)m_pTwainInterface->DSM_Lock((TW_HANDLE)twCap.hContainer);

			if(twCap.hContainer)
			{
				if(TWTY_BOOL >= pOneValue->ItemType)
				{
					*value = pOneValue->Item;
				}
				m_pTwainInterface->DSM_Unlock((TW_HANDLE)twCap.hContainer);
			}
			break;
		}
		case TWON_ENUMERATION:
		{
			pTW_ENUMERATION pEnumeration;
			pEnumeration = (pTW_ENUMERATION)m_pTwainInterface->DSM_Lock((TW_HANDLE)twCap.hContainer);
			if(twCap.hContainer)
			{
				if(TWTY_BOOL >= pEnumeration->ItemType)
				{
					*value = pEnumeration->ItemList[pEnumeration->CurrentIndex*GetTWTypeSize(pEnumeration->ItemType)];
				}
				m_pTwainInterface->DSM_Unlock((TW_HANDLE)twCap.hContainer);
			}
			break;
		}
		case TWON_ARRAY:
		default:
		{
			bEnable = FALSE;
			break;
		}
	}

	m_pTwainInterface->DSM_Free((TW_HANDLE)twCap.hContainer);

	return bEnable;
}

BOOL CapabilityInterface::GetCapArray(TW_UINT16 cap, TW_UINT16 twType, void *otList, TW_UINT32 *listnum)
{
	BOOL				bEnable		= TRUE;
	TW_UINT16			twrc		= TWRC_SUCCESS;
	TW_UINT32			itemNum		= 0;
	TW_CAPABILITY		twCap;
	

	twCap.Cap			= cap;
	twCap.ConType		= TWON_DONTCARE16;
	twCap.hContainer	= NULL;

	twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
										DAT_CAPABILITY,
										MSG_GET,
										(TW_MEMREF)&twCap,
										TRUE);

	if(TWRC_SUCCESS != twrc)
		return FALSE;

	pTW_ARRAY pGetArray = (pTW_ARRAY)m_pTwainInterface->DSM_Lock((TW_HANDLE)twCap.hContainer);

	switch(twCap.ConType)
	{
		case TWON_ARRAY:
		{
			if(*listnum > pGetArray->NumItems)
				itemNum = pGetArray->NumItems;
			else
				itemNum = *listnum;
				
			memcpy(otList, pGetArray->ItemList, (itemNum * GetTWTypeSize(twType)));

			break;
		}

		default:
		{
			bEnable = FALSE;
			break;
		}
	}

	*listnum = itemNum;
	
	m_pTwainInterface->DSM_Unlock((TW_HANDLE)twCap.hContainer);
	m_pTwainInterface->DSM_Free((TW_HANDLE)twCap.hContainer);

	return bEnable;
}

//========================================================================
BOOL CapabilityInterface::SetOneValue(TW_UINT16 cap, TW_UINT16 twType, const TW_UINT32 *inData)
{
	BOOL bEnable = FALSE;
	TW_CAPABILITY	twCap;
	TW_UINT16		twrc		= TWRC_SUCCESS;

	twCap.Cap = cap;
	twCap.ConType = TWON_ONEVALUE;

	twCap.hContainer =  m_pTwainInterface->DSM_Alloc(sizeof(TW_ONEVALUE));
	pTW_ONEVALUE pOneValue = (pTW_ONEVALUE)m_pTwainInterface->DSM_Lock(twCap.hContainer);
	pOneValue->ItemType = twType;
	pOneValue->Item = *inData;

	m_pTwainInterface->DSM_Unlock(twCap.hContainer);
	twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
										DAT_CAPABILITY,
										MSG_SET,
										(TW_MEMREF)&twCap,
										TRUE);

	m_pTwainInterface->DSM_Free(twCap.hContainer);

	return bEnable;
}

//========================================================================
BOOL CapabilityInterface::SetFrame(TW_UINT16 cap, const TW_FRAME *inFrame)
{
	BOOL bEnable = FALSE;
	TW_CAPABILITY	twCap;
	TW_UINT16		twrc		= TWRC_SUCCESS;

	twCap.Cap = cap;
	twCap.ConType = TWON_ONEVALUE;

	twCap.hContainer =  m_pTwainInterface->DSM_Alloc(sizeof(TW_ONEVALUE)+sizeof(TW_FRAME));
	pTW_ONEVALUE pOneValue = (pTW_ONEVALUE)m_pTwainInterface->DSM_Lock(twCap.hContainer);
	pOneValue->ItemType = TWTY_FRAME;
	memcpy(&pOneValue->Item, inFrame, sizeof(TW_FRAME));

	m_pTwainInterface->DSM_Unlock(twCap.hContainer);
	twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
										DAT_CAPABILITY,
										MSG_SET,
										(TW_MEMREF)&twCap,
										TRUE);

	m_pTwainInterface->DSM_Free(twCap.hContainer);

	return bEnable;
}

//========================================================================
BOOL	CapabilityInterface::SetCapArray(TW_UINT16 cap, TW_UINT16 twType, const void *inList, TW_UINT32 listnum)
{
	BOOL bEnable = FALSE;
	TW_CAPABILITY	twCap;
	TW_UINT16		twrc		= TWRC_SUCCESS;

	twCap.Cap = cap;
	twCap.ConType = TWON_ARRAY;

	twCap.hContainer =  m_pTwainInterface->DSM_Alloc(sizeof(TW_ARRAY) + (GetTWTypeSize(twType) * listnum));
	pTW_ARRAY pArray = (pTW_ARRAY)m_pTwainInterface->DSM_Lock(twCap.hContainer);
	pArray->ItemType = twType;
	pArray->NumItems = listnum;
	memcpy(pArray->ItemList, inList, (GetTWTypeSize(twType) *listnum));
				
	m_pTwainInterface->DSM_Unlock(twCap.hContainer);
	twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
										DAT_CAPABILITY,
										MSG_SET,
										(TW_MEMREF)&twCap,
										TRUE);

	m_pTwainInterface->DSM_Free(twCap.hContainer);

	return bEnable;
	
}

//========================================================================
TW_UINT16 CapabilityInterface::GetTWTypeSize(TW_UINT16 type)
{
	TW_UINT16 size = 0;

	switch(type)
	{
		case TWTY_INT8:
			size = sizeof(TW_INT8);
			break;
		case TWTY_INT16:
			size = sizeof(TW_INT16);
			break;
		case TWTY_INT32:
			size = sizeof(TW_INT32);
			break;
		case TWTY_UINT8:
			size = sizeof(TW_UINT8);
			break;
		case TWTY_UINT16:
			size = sizeof(TW_UINT16);
			break;
		case TWTY_UINT32:
			size = sizeof(TW_UINT32);
			break;
		case TWTY_BOOL:
			size = sizeof(TW_BOOL);
			break;
		case TWTY_FIX32:
			size = sizeof(TW_FIX32);
			break;
		case TWTY_FRAME:
			size = sizeof(TW_FRAME);
			break;
		case TWTY_STR32:
			size = sizeof(TW_STR32);
			break;
		case TWTY_STR64:
			size = sizeof(TW_STR64);
			break;
		case TWTY_STR128:
			size = sizeof(TW_STR128);
			break;
		case TWTY_STR255:
			size = sizeof(TW_STR255);
			break;
	}

	return size;
}