#include "stdafx.h"
#include "DataManager.h"

//========================================================================
DataManager::DataManager(void)
{
	SetRecogDataState(RECOGDATA_MICR, RECOG_DISABLE);
	SetRecogDataState(RECOGDATA_OCR, RECOG_DISABLE);
	SetRecogDataState(RECOGDATA_BARCODE, RECOG_DISABLE);
	ResetRecogData();

	m_hBitmap			= NULL;
}

//========================================================================
DataManager::~DataManager(void)
{
	if(m_hBitmap)
	{
		GlobalFree((HGLOBAL)m_hBitmap);
		m_hBitmap = NULL;
	}

	ResetRecogData();
}

//========================================================================
RECOG_STATE DataManager::GetDecodeMicr(EPMICR_DATA *micr)
{
	if(RECOG_DECORD == m_MicrState)
	{
		*micr = m_dataMicr;
	}

	return m_MicrState;
}

//========================================================================
RECOG_STATE DataManager::GetDecodeOcr(EPOCR_DATA *ocr)
{
	if(RECOG_DECORD == m_OcrState)
	{
		*ocr = m_dataOcr;
	}

	return m_OcrState;
}

//========================================================================
RECOG_STATE DataManager::GetDecodeBarcode(EPBarcodeDataList *barcode)
{
	if(RECOG_DECORD == m_BarcodeState)
	{
		barcode->clear();
		for(UINT i = 0; i < m_dataBarcode.size(); i++)
		{
			barcode->push_back(m_dataBarcode.at(i));
		}
	}

	return m_BarcodeState;
}

//========================================================================
void DataManager::SetBitmapHandle(BITMAPINFOHEADER *pBIH)
{
	BOOL				bSuccess	= TRUE;
	DWORD				dwSize		= 0;
	BITMAPINFOHEADER	*pBIHTmp		= NULL;

	if(!pBIH)
	{
		bSuccess = FALSE;
	}

	if(bSuccess)
	{
		dwSize		=  sizeof(BITMAPINFOHEADER)				+
					   (pBIH->biClrUsed * sizeof(RGBQUAD))	+
					   ((((pBIH->biWidth * pBIH->biBitCount + 31)/32) * 4) * pBIH->biHeight);

		m_hBitmap	= (HBITMAP)GlobalAlloc(GHND, dwSize);

		if(!m_hBitmap)
		{
			bSuccess = FALSE;
		}
	}

	if(bSuccess)
	{
		pBIHTmp = (BITMAPINFOHEADER *)GlobalLock((HGLOBAL)m_hBitmap);
		memcpy_s(pBIHTmp, dwSize, pBIH, dwSize);
		GlobalUnlock((HGLOBAL)m_hBitmap);
	}
}

//========================================================================
HBITMAP DataManager::GetBitmapHandle(void)
{
	return m_hBitmap;
}

//========================================================================
void DataManager::FreeBitmapHandle(void)
{
	if(m_hBitmap)
	{
		GlobalFree(m_hBitmap);
		m_hBitmap = NULL;
	}
}

//========================================================================
void DataManager::SetDecodeMicrData(EPMICR_DATA *pData, size_t size)
{
	memcpy_s(&m_dataMicr, sizeof(m_dataMicr), pData, size);
	SetRecogDataState(RECOGDATA_MICR, RECOG_DECORD);
}

//========================================================================
void DataManager::SetDecodeOcrData(EPOCR_DATA *pData, size_t size)
{
	memcpy_s(&m_dataOcr, sizeof(m_dataOcr), pData, size);
	SetRecogDataState(RECOGDATA_OCR, RECOG_DECORD);
}

//========================================================================
void DataManager::SetDecodeBarcodeData(EPBarcodeDataList *pData)
{
	std::vector<EPBARCODE_DATA>::iterator it = pData->begin();

	for(; it != pData->end(); it++)
	{
		m_dataBarcode.push_back((EPBARCODE_DATA)(*it));
	}
	SetRecogDataState(RECOGDATA_BARCODE, RECOG_DECORD);
}

//========================================================================
void DataManager::ResetRecogData(void)
{
	ResetMicrData();
	ResetOcrData();
	ResetBarcodeData();
}

//========================================================================
void DataManager::ResetMicrData(void)
{
	memset(&m_dataMicr, NULL, sizeof(m_dataMicr));
	SetRecogDataState(RECOGDATA_MICR, RECOG_DISABLE);
}

//========================================================================
void DataManager::ResetOcrData(void)
{
	memset(&m_dataOcr, NULL, sizeof(m_dataOcr));
	SetRecogDataState(RECOGDATA_OCR, RECOG_DISABLE);
}

//========================================================================
void DataManager::ResetBarcodeData(void)
{
	m_dataBarcode.clear();
	SetRecogDataState(RECOGDATA_BARCODE, RECOG_DISABLE);
}

//========================================================================
void DataManager::SetRecogDataState(TW_UINT32 recogType, RECOG_STATE state)
{
	switch(recogType)
	{
		case RECOGDATA_MICR:
			m_MicrState = state;
			break;

		case RECOGDATA_OCR:
			m_OcrState = state;
			break;

		case RECOGDATA_BARCODE:
			m_BarcodeState = state;
			break;
	}
}

