#include "stdafx.h"
#include "SampleApp.h"
#include "TransferManager.h"

//========================================================================
TransferManager::TransferManager(HWND hDlg, DataManager *pDataManager, TwainInterface *pTwainInterface)
{
	m_hDlg				= hDlg;
	m_pDataManager		= pDataManager;
	m_pTwainInterface	= pTwainInterface;

	m_bEnableMICR		= FALSE;
	m_bEnableOCR		= FALSE;
	m_bEnableBarcode	= FALSE;
}

//========================================================================
TransferManager::~TransferManager(void)
{
	;
}

//========================================================================
void TransferManager::NativeTransfer(void)
{
	TW_PENDINGXFERS     twPendingXfer;
	TW_UINT32           hBitMap = NULL;

	do
	{
		TW_UINT16	twrc = TWRC_FAILURE;

		CheckEnableRecogData();

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_IMAGENATIVEXFER,
											MSG_GET,
											&hBitMap);

		memset(&twPendingXfer, 0, sizeof(TW_PENDINGXFERS));

		switch(twrc)
		{
			case TWRC_XFERDONE:  // Session is in State 7

				ResetRecogData();
				StoreRecogData();

				twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
													DAT_PENDINGXFERS,
													MSG_ENDXFER,
													&twPendingXfer);

				if((HBITMAP)IntToPtr(hBitMap) >= (HANDLE)VALID_HANDLE)
				{
					TW_MEMREF	hBitMapTmp	= NULL;
					hBitMapTmp = (TW_MEMREF)m_pTwainInterface->DSM_Lock((TW_HANDLE)IntToPtr(hBitMap));
					m_pDataManager->SetBitmapHandle((BITMAPINFOHEADER *)hBitMapTmp);
					m_pTwainInterface->DSM_Unlock((TW_HANDLE)IntToPtr(hBitMap));
					m_pTwainInterface->DSM_Free((TW_HANDLE)IntToPtr(hBitMap));
					hBitMap = NULL;
				}

				break;

			case TWRC_CANCEL:   // Session is in State 7

				twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
													DAT_PENDINGXFERS,
													MSG_ENDXFER,
													&twPendingXfer);

				break;

			case TWRC_FAILURE:  // Session is in State 6

				twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
													DAT_PENDINGXFERS,
													MSG_ENDXFER,
													&twPendingXfer);

			default:			// Sources should never return any other RC

				twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
													DAT_PENDINGXFERS,
													MSG_ENDXFER,
													&twPendingXfer);

		}

		SendMessage(m_hDlg, WM_XFERDATA, NULL, NULL);

	}
	while(twPendingXfer.Count != 0);

	SendMessage(m_hDlg, WM_ENDXFER, NULL, NULL);
	m_pTwainInterface->disableDS(m_hDlg);

}

//========================================================================
void TransferManager::FileTransfer(void)
{
}

//========================================================================
void TransferManager::MemTransfer(void)
{
}

//========================================================================
void TransferManager::CheckEnableRecogData(void)
{
	m_bEnableMICR		= CheckRecogCap(CAP_MICRENABLED);
	m_bEnableOCR		= CheckRecogCap(ICAP_EPCS_OCRABENABLED);
	m_bEnableBarcode	= CheckRecogCap(ICAP_BARCODEDETECTIONENABLED);
}

BOOL TransferManager::CheckRecogCap(TW_UINT16 cap)
{
	BOOL				bEnable		= FALSE;
	TW_UINT16			twrc		= TWRC_SUCCESS;
	TW_CAPABILITY		twCap;

	twCap.Cap			= cap;
	twCap.ConType		= TWON_DONTCARE16;
	twCap.hContainer	= NULL;
	twrc = m_pTwainInterface->DSM_Entry(DG_CONTROL,
										DAT_CAPABILITY,
										MSG_GETCURRENT,
										(TW_MEMREF)&twCap,
										FALSE);

	if(TWRC_SUCCESS == twrc)
	{
		TW_ONEVALUE	*pOneValue	= NULL;

		pOneValue = (TW_ONEVALUE *)m_pTwainInterface->DSM_Lock((TW_HANDLE)twCap.hContainer);

		if((TWTY_BOOL == pOneValue->ItemType) &&
				(TRUE == pOneValue->Item))
		{
			bEnable = TRUE;
		}
	}

	if(twCap.hContainer)
	{
		m_pTwainInterface->DSM_Unlock((TW_HANDLE)twCap.hContainer);
		m_pTwainInterface->DSM_Free((TW_HANDLE)twCap.hContainer);
		twCap.hContainer = NULL;
	}

	return bEnable;
}

//========================================================================
void TransferManager::StoreRecogData(void)
{
	TW_UINT16			twrc			= TWRC_SUCCESS;
	TW_EXTIMAGEINFO		infoExtImg;

	// Get MICR data
	if(m_bEnableMICR)
	{
		BOOL	bDecord	= TRUE;

		EPMICR_DATA		micrData;
		memset(&micrData, NULL, sizeof(micrData));
		memset(&infoExtImg, NULL, sizeof(TW_EXTIMAGEINFO));

		m_pDataManager->SetRecogDataState(RECOGDATA_MICR, RECOG_ENABLE);

		// TWEI_MAGTYPE
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_MAGTYPE;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_UINT16  == infoExtImg.Info[0].ItemType))
			{

				micrData.magType = (TW_UINT16)infoExtImg.Info[0].Item;
			}

			if(infoExtImg.Info[0].NumItems > 1)
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		// TWEI_MAGDATA
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_MAGDATA;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_STR255  == infoExtImg.Info[0].ItemType))
			{
				TW_STR255 *pItem = (TW_STR255 *)m_pTwainInterface->DSM_Lock((TW_HANDLE)infoExtImg.Info[0].Item);
				strcpy_s(micrData.magData, sizeof(micrData.magData), *pItem);

				m_pTwainInterface->DSM_Unlock((TW_HANDLE)infoExtImg.Info[0].Item);
			}

			if((bDecord) || (infoExtImg.Info[0].NumItems > 1))
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		if(bDecord)
		{
			m_pDataManager->SetDecodeMicrData(&micrData, sizeof(micrData));
		}
	}

	// Get OCR data
	if(m_bEnableOCR)
	{
		BOOL	bDecord	= TRUE;

		EPOCR_DATA		ocrData;
		memset(&ocrData, 0, sizeof(ocrData));
		memset(&infoExtImg, NULL, sizeof(TW_EXTIMAGEINFO));

		m_pDataManager->SetRecogDataState(RECOGDATA_OCR, RECOG_ENABLE);

		// TWEI_EP_OCRABDATA
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_EPCS_OCRABDATA;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_STR255  == infoExtImg.Info[0].ItemType))
			{
				TW_STR255 *pItem = (TW_STR255 *)m_pTwainInterface->DSM_Lock((TW_HANDLE)infoExtImg.Info[0].Item);
				strcpy_s(ocrData.Data, sizeof(ocrData.Data), *pItem);

				m_pTwainInterface->DSM_Unlock((TW_HANDLE)infoExtImg.Info[0].Item);
			}

			if((bDecord) || (infoExtImg.Info[0].NumItems > 1))
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		if(bDecord)
		{
			m_pDataManager->SetDecodeOcrData(&ocrData, sizeof(ocrData));
		}
	}

	// Get Barcode data
	if(m_bEnableBarcode)
	{
		BOOL				bDecord		= TRUE;
		TW_UINT32			count		= 0;
		EPBarcodeDataList	barDataList;
		std::vector<EPBARCODE_DATA>::iterator it;
		memset(&infoExtImg, NULL, sizeof(TW_EXTIMAGEINFO));

		m_pDataManager->SetRecogDataState(RECOGDATA_BARCODE, RECOG_ENABLE);

		// TWEI_BARCODECOUNT
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_BARCODECOUNT;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_UINT32  == infoExtImg.Info[0].ItemType))
			{
				count = (TW_UINT32)infoExtImg.Info[0].Item;
			}

			if(infoExtImg.Info[0].NumItems > 1)
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		// Create list
		for(TW_UINT32 i=0; i<count; i++)
		{
			BarcodeDataT	barcode;
			memset(&barcode, NULL, sizeof(BarcodeDataT));
			barDataList.push_back(barcode);
		}

		// TWEI_BARCODEROTATION
		memset(&infoExtImg, NULL, sizeof(TW_EXTIMAGEINFO));
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_BARCODEROTATION;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_UINT32  == infoExtImg.Info[0].ItemType))
			{
				it = barDataList.begin();
				if(infoExtImg.Info[0].NumItems == 1)
				{
					(*it).Rotation = (TW_UINT32)infoExtImg.Info[0].Item;
				}
				else
				{
					TW_UINT32 *pItem = (TW_UINT32 *)m_pTwainInterface->DSM_Lock((TW_HANDLE)infoExtImg.Info[0].Item);
					for(TW_UINT32 i=0; i<count, i<infoExtImg.Info[0].NumItems; i++, it++)
					{
						if(pItem[i])
						{
							(*it).Rotation = pItem[i];
						}
					}
					m_pTwainInterface->DSM_Unlock((TW_HANDLE)infoExtImg.Info[0].Item);
				}
			}

			if(infoExtImg.Info[0].NumItems > 1)
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		// TWEI_BARCODETEXTLENGTH
		memset(&infoExtImg, NULL, sizeof(TW_EXTIMAGEINFO));
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_BARCODETEXTLENGTH;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_UINT32  == infoExtImg.Info[0].ItemType))
			{
				it = barDataList.begin();
				if(infoExtImg.Info[0].NumItems == 1)
				{
					(*it).TextLength = (TW_UINT32)infoExtImg.Info[0].Item;
				}
				else
				{
					TW_UINT32 *pItem = (TW_UINT32 *)m_pTwainInterface->DSM_Lock((TW_HANDLE)infoExtImg.Info[0].Item);
					for(TW_UINT32 i=0; i<count, i<infoExtImg.Info[0].NumItems; i++, it++)
					{
						if(pItem[i])
						{
							(*it).TextLength = pItem[i];
						}
					}
					m_pTwainInterface->DSM_Unlock((TW_HANDLE)infoExtImg.Info[0].Item);
				}
			}

			if(infoExtImg.Info[0].NumItems > 1)
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		// TWEI_BARCODETEXT
		memset(&infoExtImg, NULL, sizeof(TW_EXTIMAGEINFO));
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_BARCODETEXT;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_STR255  == infoExtImg.Info[0].ItemType))
			{
				it = barDataList.begin();
				TW_STR255 *pItem = (TW_STR255 *)m_pTwainInterface->DSM_Lock((TW_HANDLE)infoExtImg.Info[0].Item);

				for(TW_UINT32 i=0; i<count, i<infoExtImg.Info[0].NumItems; i++, it++)
				{
					if(pItem[i])
					{
						strcpy_s((*it).Text, sizeof(TW_STR255)-1, pItem[i]);
					}
				}
				m_pTwainInterface->DSM_Unlock((TW_HANDLE)infoExtImg.Info[0].Item);
			}

			if((bDecord) || (infoExtImg.Info[0].NumItems > 1))
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		// TWEI_BARCODEX
		memset(&infoExtImg, NULL, sizeof(TW_EXTIMAGEINFO));
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_BARCODEX;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_UINT32  == infoExtImg.Info[0].ItemType))
			{
				it = barDataList.begin();
				if(infoExtImg.Info[0].NumItems == 1)
				{
					(*it).BarcodeX = (TW_UINT32)infoExtImg.Info[0].Item;
				}
				else
				{
					TW_UINT32 *pItem = (TW_UINT32 *)m_pTwainInterface->DSM_Lock((TW_HANDLE)infoExtImg.Info[0].Item);
					for(TW_UINT32 i=0; i<count, i<infoExtImg.Info[0].NumItems; i++, it++)
					{
						if(pItem[i])
						{
							(*it).BarcodeX = pItem[i];
						}
					}
					m_pTwainInterface->DSM_Unlock((TW_HANDLE)infoExtImg.Info[0].Item);
				}
			}

			if(infoExtImg.Info[0].NumItems > 1)
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		// TWEI_BARCODEY
		memset(&infoExtImg, NULL, sizeof(TW_EXTIMAGEINFO));
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_BARCODEY;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_UINT32  == infoExtImg.Info[0].ItemType))
			{
				it = barDataList.begin();
				if(infoExtImg.Info[0].NumItems == 1)
				{
					(*it).BarcodeY = (TW_UINT32)infoExtImg.Info[0].Item;
				}
				else
				{
					TW_UINT32 *pItem = (TW_UINT32 *)m_pTwainInterface->DSM_Lock((TW_HANDLE)infoExtImg.Info[0].Item);
					for(TW_UINT32 i=0; i<count, i<infoExtImg.Info[0].NumItems; i++, it++)
					{
						if(pItem[i])
						{
							(*it).BarcodeY = pItem[i];
						}
					}
					m_pTwainInterface->DSM_Unlock((TW_HANDLE)infoExtImg.Info[0].Item);
				}
			}

			if(infoExtImg.Info[0].NumItems > 1)
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		// TWEI_BARCODETYPE
		memset(&infoExtImg, NULL, sizeof(TW_EXTIMAGEINFO));
		infoExtImg.NumInfos			= 1;
		infoExtImg.Info[0].InfoID	= TWEI_BARCODETYPE;

		twrc = m_pTwainInterface->DSM_Entry(DG_IMAGE,
											DAT_EXTIMAGEINFO,
											MSG_GET,
											&infoExtImg);

		if(TWRC_SUCCESS == twrc)
		{
			if((TWRC_SUCCESS == infoExtImg.Info[0].CondCode)	&&
					(TWTY_UINT32  == infoExtImg.Info[0].ItemType))
			{
				it = barDataList.begin();
				if(infoExtImg.Info[0].NumItems == 1)
				{
					(*it).Type = (TW_UINT32)infoExtImg.Info[0].Item;
				}
				else
				{
					TW_UINT32 *pItem = (TW_UINT32 *)m_pTwainInterface->DSM_Lock((TW_HANDLE)infoExtImg.Info[0].Item);
					for(TW_UINT32 i=0; i<count, i<infoExtImg.Info[0].NumItems; i++, it++)
					{
						if(pItem[i])
						{
							(*it).Type = pItem[i];
						}
					}
					m_pTwainInterface->DSM_Unlock((TW_HANDLE)infoExtImg.Info[0].Item);
				}
			}

			if(infoExtImg.Info[0].NumItems > 1)
			{
				m_pTwainInterface->DSM_Free((TW_HANDLE)infoExtImg.Info[0].Item);
			}
		}

		if((TWRC_SUCCESS != twrc) || (TWRC_SUCCESS != infoExtImg.Info[0].CondCode))
		{
			bDecord = FALSE;
		}

		if(bDecord)
		{
			m_pDataManager->SetDecodeBarcodeData(&barDataList);
		}
	}
}

//========================================================================
void TransferManager::ResetRecogData(void)
{
	m_pDataManager->ResetRecogData();
}

